/******************************************************************************
  KnitBox, controller for manual Knitting Machine. Arduino Nano
  Using hall sensor or reed relay to detect row and sled direction. Increase/decrease can be calculated nd indicated during process.

  Display Shield 16x2 char LCD display with Keypad "Select, Left, Right, Up, Down" on Pin A0, plus reset button
  Display connector to pins 8, 9, 4, 5, 6, 7
  5V
  GND

  Hall Switches for counting carriage movements
  Left Hall Sw to pin 2
  Right Hall Sw to pin 3
  5V
  GND

  Piezo Speaker to buzzer-pin 11

  Star Wars Melody taken from: https://github.com/robsoncouto/arduino-songs

******************************************************************************/

#include <LiquidCrystal.h>  // 16x2 LCD library
#include <OneButton.h>      // used for Hall sensors
#define debug
int maxLoops = 220;         // max number of loops

uint8_t Lge = 0;            // Language selector - 0=English, 1=German, 2=???, 3=??? - See below for displayed text
const uint8_t DispCol = 4;  // Room for 4 langusges, can be extended if desired

const char *const DispTxt[][DispCol] = {
  // adjust columns text for display indexed by language variable 'Lge'
  // keep individual cell length as defined per row, just change text (no special characters in this default font)
  { " Loops Start", " Maschen Anf", "------------", "------------" },                  //  0 number of loops first row (12 chars, right aligned)
  { "        Rows", "      Reihen", "------------", "------------" },                  //  1 number of rows (12 chars, right aligned)
  { "   Loops End", "Maschen Ziel", "------------", "------------" },                  //  2 number of loops target (12 chars, right aligned)
  { "Pattern", "   Form", "-------", "-------" },                                      //  3 desired shape/pattern (7 chars, right aligned)
  { "   Ready, Go!   ", "   Los geht's   ", "----------------", "----------------" },  //  4 Start knitting message (16 chars, one full line)
  { "Drop  ", "Abnhmn", "------", "------" },                                          //  5 remove number of loops (6 chars)
  { "Add   ", "Zunhmn", "------", "------" },                                          //  6 add number of loops (6 chars)
  { " Done     ", " Fertig   ", "----------", "----------" },                          //  7 we're done (10 chars)
  { "Lft-Al", "Lin-Bd", "------", "------" },                                          //  8 Left Justified (6 chars)
  { "Center", "Mittig", "------", "------" },                                          //  9 Center aligned (6 chars)
  { "Rgt-Al", "Rec-Bd", "------", "------" },                                          // 10 Right justified  (6 chars)
  { "Knitting Counter", "  Strickzaehler", "", "" },                                   // 11 Welcome Message, max 16 chars
  { "L-Offs", "L-Vers", "------", "------" },                                          // 12 Left-Aligned-Offset (6 chars)
  { "R", "R", "-", "-" },                                                              // 13 "R"ows  (1 char)
  { "L", "M", "-", "-" },                                                              // 14 "L"oops (1 char)
  { "O", "V", "-", "-" },                                                              // 15 "O"ffset (1 char)
  { "D", "R", "-", "-" },                                                              // 16 sled "D"irection (left/right) (1 char)
  { "r", "r", "-", "-" },                                                              // 17 "r"ight (1 char)
  { "l", "l", "-", "-" },                                                              // 18 "l"eft (1 char)
};
//const uint8_t DispRows = sizeof(DispTxt) / sizeof(DispTxt[0]);  // get number of rows from DispTxt array


// select the pins used for the LCD panel
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);  // see wiring diagram

// define some values used by the panel and buttons (resistor network, single line analog connections)
// If buttons don't work as intended, adjust analog read values in routine "read_LCD_buttons"
#define btnRIGHT 0
#define btnUP 1
#define btnDOWN 2
#define btnLEFT 3
#define btnSELECT 4
#define btnNONE 5

// definition of pins for other components
#define buzzer 11  // buzzer pin
#define HallL 2    // Hall Switch Left Pin (if the final build has left/right swapped and display showing "D:" direction the wrong way, flip these two pins)
#define HallR 3    // Hall Switch Right Pin

int lcd_key = 0;
int adc_key_in = 0;
bool SensorL = false;   // Hall Sensor left active/inactive
bool SensorR = false;   // Hall Sensor right
bool DirLeft = false;   // true = left
int cnum = 0;           // collected 3-digit number
int LoopNumStart = 0;   // Start number of loops
int LoopNumTarget = 0;  // Target number of loops
int Rows = 0;           // Number of rows
int KnitForm = 0;       // 0 = Lft (left aligned pattern), 1 = Ctr (even left+right increase/decrese), 2 = Rgt (right aligned pattern), 3 = Lof (Left Offset relative to start)
int Rcounter = 0;       // Rows counter
int ORcounter = 0;      // Original/Previous Rcounter value
float LoopSlope = 0;    // Loop slope value for add/removal of loops
int LeftOffset = 0;     // Left target offset relative to left start if free form
float LofSlope = 0;     // Offset value freeform
int Lpos = 0;           // Left position relative to the left start
int Rpos = 0;           // Right position calculated from Lpos + number of loops

// make extra custom character patterns:
// right/left arrow is part of the standard Font 7E/7F
byte b1[8] = {  // up arrow
  0b00100,
  0b01110,
  0b10101,
  0b00100,
  0b00100,
  0b00000,
  0b00000,
  0b00000
};
byte b2[8] = {  // down arrow
  0b00000,
  0b00000,
  0b00000,
  0b00100,
  0b00100,
  0b10101,
  0b01110,
  0b00100
};
byte b3[8] = {  // up-down arrow
  0b00100,
  0b01110,
  0b10101,
  0b00100,
  0b00100,
  0b10101,
  0b01110,
  0b00100
};
byte b4[8] = {  // Left aligned box (left oriented pattern)
  0b11000,
  0b10100,
  0b10010,
  0b10001,
  0b11111,
  0b00000,
  0b00000,
  0b00000
};
byte b5[8] = {  // Center box (Equal reduction/increase)
  0b00100,
  0b01010,
  0b01010,
  0b10001,
  0b10001,
  0b11111,
  0b00000,
  0b00000
};
byte b6[8] = {  // Right aligned box (right oriented pattern)
  0b00011,
  0b00101,
  0b01001,
  0b10001,
  0b11111,
  0b00000,
  0b00000,
  0b00000
};

// Final Melody setup
// Star Wars Theme Notes/frequency
#define NOTE_F5 698
#define NOTE_C6 1047
#define NOTE_AS5 932
#define NOTE_A5 880
#define NOTE_G5 784
#define NOTE_F6 1397

// change this to make the song slower or faster
int tempo = 135;  // org 108

int melody[] = {
  // to be played once target row has been reached
  NOTE_AS5,
  8,
  NOTE_A5,
  8,
  NOTE_G5,
  8,
  NOTE_F6,
  2,
  NOTE_C6,
  4,
  NOTE_AS5,
  8,
  NOTE_A5,
  8,
  NOTE_G5,
  8,
  NOTE_F6,
  2,
  NOTE_C6,
  4,
  NOTE_AS5,
  8,
  NOTE_A5,
  8,
  NOTE_AS5,
  8,
  NOTE_G5,
  2,
};

// define melody as listed above
int notes = sizeof(melody) / sizeof(melody[0]) / 2;
int wholenote = (60000 * 4) / tempo;
int divider = 0, noteDuration = 0;


void setup() {
  Serial.begin(115200);         // start serial monitor
  lcd.begin(16, 2);             // start the display library
  lcd.setCursor(0, 0);          // Start top left
  lcd.print(DispTxt[11][Lge]);  // print a simple welcome message
  delay(1000);
  // create special characters
  lcd.createChar(1, b1);  // up arrow
  lcd.createChar(2, b2);  // dn arrow
  lcd.createChar(3, b3);  // up-dn arrow
  lcd.createChar(4, b4);  // left aligned box
  lcd.createChar(5, b5);  // certer box
  lcd.createChar(6, b6);  // right aligned box

  pinMode(HallL, INPUT_PULLUP);  // left Hall Sensor
  pinMode(HallR, INPUT_PULLUP);  // right Hall Sensor
  attachInterrupt(digitalPinToInterrupt(HallL), HallLticks, FALLING);
  attachInterrupt(digitalPinToInterrupt(HallR), HallRticks, FALLING);

  // Play power-up tones
  tone(buzzer, 800);
  delay(500);
  noTone(buzzer);
  delay(100);
  tone(buzzer, 1000);
  delay(100);
  noTone(buzzer);
  delay(100);
  tone(buzzer, 1500);
  delay(100);
  noTone(buzzer);
  delay(100);

  // Get starting number of loops
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(DispTxt[0][Lge]);  // Number of loops at start
  lcd.print(":000");
  lcd.setCursor(0, 1);
  lcd.print("s");
  lcd.print(char(0x7f));  // left arrow
  lcd.write(3);           // up/dn arrow
  lcd.print(char(0x7e));  // right arrow
  LoopNumStart = collectNumber();
  Serial.print("Loops Start: ");  // verify on serial monitor
  Serial.println(LoopNumStart);

  // Number of rows
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(DispTxt[1][Lge]);
  lcd.print(":000 ");
  lcd.setCursor(0, 1);
  lcd.print("s");
  lcd.print(char(0x7f));  // left arrow
  lcd.write(3);           // up/dn arrow
  lcd.print(char(0x7e));  // right arrow
  Rows = collectNumber();
  Serial.print("Rows: ");
  Serial.println(Rows);

  // Target number of loops
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(DispTxt[2][Lge]);
  lcd.print(":000 ");
  lcd.setCursor(0, 1);
  lcd.print("s");
  lcd.print(char(0x7f));  // left arrow
  lcd.write(3);           // up/dn arrow
  lcd.print(char(0x7e));  // right arrow
  LoopNumTarget = collectNumber();
  Serial.print("Loops Target: ");
  Serial.println(LoopNumTarget);

  // choose form, left aligned, centered, right aligned, left offset for more flexible form
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(DispTxt[3][Lge]);  // Pattern
  lcd.print(":");
  lcd.print(DispTxt[8][Lge]);
  lcd.print(" ");
  lcd.write(4);  // pattern icon
  lcd.setCursor(0, 1);
  lcd.print("s");
  lcd.write(3);   // up/dn arrow
  collectForm();  // basic pattern orientation
  Serial.print("Pattern: ");
  Serial.println(DispTxt[8][Lge]);

  // Calculate slopes left/right
  Rcounter = 0;
  CalcSlopes();

  // Display start message and play tones
  lcd.setCursor(0, 1);
  lcd.print(DispTxt[4][Lge]);
  tone(buzzer, 800);
  delay(500);
  noTone(buzzer);
  delay(300);
  tone(buzzer, 1000, 100);
  delay(300);
  tone(buzzer, 1200, 100);
  delay(300);
  tone(buzzer, 1400, 100);
  delay(300);

  // clear Sensor readings if any are left over from testing/debug during data entry
  SensorL = false;
  SensorR = false;
}

void HallLticks() {  // Left Hall Sensor Interrupt routine
  SensorL = true;
#ifdef debug
  tone(buzzer, 1200, 20);
#endif
  if (SensorR) {  // if other sensor was already triggered we know the direction
    DirLeft = true;
  }
}

void HallRticks() {  // Right Hall Sensor Interrupt routine
  SensorR = true;
#ifdef debug
  tone(buzzer, 1800, 20);
#endif
  if (SensorL) {
    DirLeft = false;
  }
}

void loop() {

  if (SensorL && SensorR)  // passed by both sensors, can clear sensor status and calculate actions
  {
    SensorL = false;
    SensorR = false;
    Rcounter++;
  }
  Radjust();  // allowing change of actual row if need be
  if (Rcounter != ORcounter) {
    InfoScreen();  // Show current data upper line and calculate Lpos/Rpos/Offset/Alerts
    ORcounter = Rcounter;
  }
}

void Radjust() {          // if change of current row needed during knitting after an issue withg dropped loops or similar
  int old_key = btnNONE;  // did key press change
loop:
  lcd_key = read_LCD_buttons();  // read the buttons
  if (lcd_key == btnNONE) {
    return;
  }
  if (old_key == lcd_key) {
    delay(10);
    goto loop;
  } else {
    old_key = lcd_key;  // remember new status avoiding to loop while key is pressed
  }

  switch (lcd_key)  // up/down button adjusts current row if something happened and a row needs to be redone or skipped
  {
    case btnUP:
      {
        Rcounter++;
        break;
      }
    case btnDOWN:
      {
        Rcounter--;
        if (Rcounter < 0) {
          Rcounter = 0;
        }
        break;
      }
    case btnNONE:
      {
        delay(10);
        break;
      }
  }
  goto loop;
}


void collectForm() {
  while (lcd_key == btnSELECT) {
    lcd_key = read_LCD_buttons();  // read the buttons
    delay(10);                     // avoid false confirmation from previous run still having select pressed
  }
  int old_key = 0;  // did key press change
  lcd.setCursor(13, 1);
  lcd.print("   ");
loop:
  lcd_key = read_LCD_buttons();  // read the buttons
  if (old_key == lcd_key) {
    delay(10);
    goto loop;
  } else {
    old_key = lcd_key;  // remember new status avoiding to loop while key is pressed
  }


  switch (lcd_key)  // depending on which button was pushed, we perform an action
  {
    case btnRIGHT:  // unused
      {
        break;
      }
    case btnLEFT:  // unused
      {
        break;
      }
    case btnUP:
      {
        KnitForm++;
        if (KnitForm > 3) {
          KnitForm = 0;
        }
        break;
      }
    case btnDOWN:
      {
        KnitForm--;
        if (KnitForm < 0) {
          KnitForm = 3;
        }
        break;
      }
    case btnSELECT:
      {
        return;
        break;
      }
    case btnNONE:
      {
        //lcd.print("NONE  ");
        delay(10);
        break;
      }
  }

  lcd.setCursor(8, 0);
  //  0 = Left-Al, 1 = Center, 2 = Right-Al, 3 = Left-Offset
  switch (KnitForm) {
    case 0:
      {
        lcd.print(DispTxt[8][Lge]);
        lcd.print(" ");
        lcd.write(4);
      }
      break;
    case 1:
      {
        lcd.print(DispTxt[9][Lge]);
        lcd.print(" ");
        lcd.write(5);
      }
      break;
    case 2:
      {
        lcd.print(DispTxt[10][Lge]);
        lcd.print(" ");
        lcd.write(6);
      }
      break;
    case 3:
      {
        lcd.print(DispTxt[12][Lge]);
        lcd.print("  ");
      }
      break;
  }
  lcd.setCursor(0, 1);
  lcd.print("s");
  lcd.write(3);
  lcd.print("    ");
  goto loop;
}

int collectNumber() {
  int cnum = 0;
  while (lcd_key == btnSELECT) {
    lcd_key = read_LCD_buttons();  // read the buttons
    delay(10);                     // avoid false confirmation from previous run still having select pressed
  }
  int old_key = 0;  // did key press change
  int pos = 13;     // start char pos on display left, 100's
  lcd.setCursor(13, 1);
  lcd.print("   ");
  lcd.setCursor(13, 1);
  lcd.write(1);

loop:
  lcd_key = read_LCD_buttons();  // read the buttons
  if (old_key == lcd_key) {
    delay(10);
    goto loop;
  } else {
    old_key = lcd_key;  // remember new status avoiding to loop while key is pressed
  }

  //Serial.print("Button: "); Serial.println(lcd_key);
  switch (lcd_key)  // depending on which button was pushed, we perform an action
  {
    case btnRIGHT:
      {
        pos++;
        if (pos > 15) {
          pos = 15;
        }
        lcd.setCursor(13, 1);
        lcd.print("   ");
        lcd.setCursor(pos, 1);
        lcd.write(1);
        break;
      }
    case btnLEFT:
      {
        pos--;
        if (pos < 13) {
          pos = 13;
        }
        lcd.setCursor(13, 1);
        lcd.print("   ");
        lcd.setCursor(pos, 1);
        lcd.write(1);
        break;
      }
    case btnUP:
      {
        if (pos == 13) {
          cnum += 100;
        }
        if (pos == 14) {
          cnum += 10;
        }
        if (pos == 15) {
          cnum += 1;
        }
        if (cnum > maxLoops) {
          cnum = maxLoops;
        }
        break;
      }
    case btnDOWN:
      {
        if (pos == 13) {
          cnum -= 100;
        }
        if (pos == 14) {
          cnum -= 10;
        }
        if (pos == 15) {
          cnum -= 1;
        }
        if (cnum < 0) {
          cnum = 0;
        }
        break;
      }
    case btnSELECT:
      {
        return cnum;
        break;
      }
    case btnNONE:
      {
        //lcd.print("NONE  ");
        delay(10);

        break;
      }
  }

  lcd.setCursor(13, 0);
  if (cnum < 100) {
    lcd.print("0");
  }
  if (cnum < 10) {
    lcd.print("0");
  }
  lcd.print(cnum);

  goto loop;
}

// function read the buttons
int read_LCD_buttons() {
  adc_key_in = analogRead(0);  // read the value from the button array
#ifdef debug
  Serial.print("Btn raw: ");
  Serial.println(adc_key_in);
  delay(50);
#endif

  // my keypad buttons showed the following readings: no button: 1020, right:0, up:101, dn:255, left:410, select:640
  // we add approx 50 to those values and check to see if we are a bit below those values. keys are listed in order of readings.
  if (adc_key_in > 900) return btnNONE;    // reading 1020 - We make this the 1st option for speed reasons since it will be the most likely result
  if (adc_key_in < 50) return btnRIGHT;    // 0
  if (adc_key_in < 150) return btnUP;      // 101
  if (adc_key_in < 310) return btnDOWN;    // 255
  if (adc_key_in < 470) return btnLEFT;    // 410
  if (adc_key_in < 690) return btnSELECT;  // 640
  return btnNONE;                          // when all others fail, return this...
}


void CalcSlopes()
// Calculate LoopSlope to determine number of loops for a particular row
// and LofSlope to determine left starting point of each row relative to the left margin of row 0.
{
  float Rtemp = Rows;
  LoopSlope = (LoopNumTarget - LoopNumStart) / Rtemp;  // Left or right adjusted ratio
  Rpos = LoopNumStart;                                 // set Rpos to initial LoopNumStart number of loops
  Serial.print("KnitForm: ");
  Serial.println(KnitForm);
  switch (KnitForm) {
    case 0:  // leftbound pattern
      {
        LofSlope = 0;
        break;
      }
    case 1:  // centered pattern
      {
        LofSlope = LoopSlope * -0.5;
        break;
      }
    case 2:  // rightbound pattern
      {
        LofSlope = LoopSlope * -1.0;
        break;
      }
    case 3:  // free offset from leftmost loop, shift positive to the right, negative to the left
      {

        LeftOffset = collectLeftOffset(LeftOffset);  // get number of loops target row is shifted
        LofSlope = LeftOffset / Rtemp;
        break;
      }
  }
  Serial.print("LoopSlope: ");
  Serial.println(LoopSlope);
  Serial.print("LofSlope: ");
  Serial.println(LofSlope);
}

int collectLeftOffset(int cnum)  // max 99 loops offset
{
  while (lcd_key == btnSELECT) {
    lcd_key = read_LCD_buttons();  // read the buttons
    delay(10);                     // avoid false confirmation from previous run still having select pressed
  }
  int old_key = 0;
  int pos = 9;  // start char pos on display left, 10's
  lcd.setCursor(pos, 1);
  lcd.print("       ");
  lcd.setCursor(pos, 1);
  lcd.write(1);
  lcd.setCursor(0, 0);
  lcd.print(DispTxt[12][Lge]);  // left offset
  lcd.print(" : 00        ");
  lcd.setCursor(0, 1);
  lcd.print(" s");
  lcd.print(char(0x7f));  // arrow left
  lcd.write(3);           // arrow up/dn
  lcd.print(char(0x7e));  // arrow right
  lcd.setCursor(0, 1);
  lcd.print(DispTxt[17][Lge]);  // move offset to the right with up arrow
  lcd.write(1);                 // arrow up
  lcd.print(" ");
  lcd.print(DispTxt[18][Lge]);  // move offset to the left with down arrow
  lcd.write(2);                 // arrow down
loop:
  lcd_key = read_LCD_buttons();  // read the buttons
  if (old_key == lcd_key) {
    delay(10);
    goto loop;
  } else {
    old_key = lcd_key;  // remember new status avoiding to loop while key is pressed
  }

  switch (lcd_key)  // depending on which button was pushed, we perform an action
  {
    case btnRIGHT:
      {
        pos++;
        if (pos > 10) {
          pos = 10;
        }
        lcd.setCursor(8, 1);
        lcd.print("   ");
        lcd.setCursor(pos, 1);
        lcd.write(1);
        break;
      }
    case btnLEFT:
      {
        pos--;
        if (pos < 9) {
          pos = 9;
        }
        lcd.setCursor(8, 1);
        lcd.print("   ");
        lcd.setCursor(pos, 1);
        lcd.write(1);
        break;
      }
    case btnUP:
      {
        if (pos == 9) {
          cnum += 10;
        }
        if (pos == 10) {
          cnum += 1;
        }
        if (cnum > 99) {
          cnum = 99;
        }
        break;
      }
    case btnDOWN:
      {
        if (pos == 9) {
          cnum -= 10;
        }
        if (pos == 10) {
          cnum -= 1;
        }
        if (cnum < -99) {
          cnum = -99;
        }
        break;
      }
    case btnSELECT:
      {
        return cnum;
        break;
      }
    case btnNONE:
      {
        delay(10);
        break;
      }
  }

  lcd.setCursor(8, 0);
  if (cnum == 0) {
    lcd.print(" ");  // clear arrow if offset=0
  }
  if (cnum > 0) {
    lcd.print(char(0x7e));  // right arrow indicating positive offset
  }
  if (cnum < 0) {
    lcd.print(char(0x7f));  // left arrow indicating negative offset
  }
  if (abs(cnum) < 10) {
    lcd.print("0");
  }
  lcd.print(abs(cnum));

  goto loop;
}

void InfoScreen()  // Display change triggered by Hall Sensor interrupt
{
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print(DispTxt[13][Lge]);  // rows
  lcd.print(":");
  lcd.print(Rcounter);
  lcd.print(" ");
  lcd.print(DispTxt[14][Lge]);  // loops
  lcd.print(":");
  lcd.print(round(LoopSlope * Rcounter + LoopNumStart));
  lcd.print(" ");
  lcd.print(DispTxt[15][Lge]);  // left offset + orientation. Moving left/right from left side of start
  lcd.print(":");
  if (Lpos > 0) {
    lcd.print(char(0x7e));  // right arrow indicating positive offset
  }
  if (Lpos < 0) {
    lcd.print(char(0x7f));  // left arrow indicating negative offset
  }
  lcd.print(abs(Lpos));
  lcd.setCursor(13, 1);
  lcd.print(DispTxt[16][Lge]);  // direction
  lcd.print(":");
  if (DirLeft) {
    lcd.print(char(0x7f));  // left arrow
  } else {
    lcd.print(char(0x7e));  // right arrow
  }
  CalcLoops();  // Calculate desired number and add/remove loops
}

void CalcLoops()  // Calculate position and number of loops depending on Rcounter
{
  int Lchange = 0;
  int Rchange = 0;
  int OldLpos = 0;
  int OldRpos = 0;
  if (DirLeft) {  // change loops only at the carriage position
    // Calculate left relative position
    OldLpos = Lpos;  // remember current position value left (relative to left start position of first row)
    Lpos = round(LofSlope * Rcounter);
    // alert left change if needed
    Lchange = Lpos - OldLpos;
    lcd.setCursor(0, 1);
    lcd.print("      ");
    if (Lchange > 0) {
      lcd.setCursor(0, 1);
      lcd.print(abs(Lchange));
      lcd.print(" ");
      lcd.print(DispTxt[5][Lge]);  // Drop Loops
      PlayDn();                    // alert user to drop loops
    }
    if (Lchange < 0) {
      lcd.setCursor(0, 1);
      lcd.print(abs(Lchange));
      lcd.print(" ");
      lcd.print(DispTxt[6][Lge]);  // Add Loops
      PlayUp();                    // alert user to add loops
    }

  } else {
    // Calculate right relative position
    OldRpos = Rpos;
    Rpos = round(LofSlope * Rcounter + LoopSlope * Rcounter + LoopNumStart);
    // alert left change if needed
    Rchange = Rpos - OldRpos;
    lcd.setCursor(0, 1);
    lcd.print("      ");
    if (Rchange < 0) {
      lcd.setCursor(0, 1);
      lcd.print(abs(Rchange));
      lcd.print(" ");
      lcd.print(DispTxt[5][Lge]);  // Drop Loops
      PlayDn();                    // alert user to drop loops
    }
    if (Rchange > 0) {
      lcd.setCursor(0, 1);
      lcd.print(abs(Rchange));
      lcd.print(" ");
      lcd.print(DispTxt[6][Lge]);  // Add Loops
      PlayUp();                    // alert user to add loops
    }
  }
  Serial.print("Rcounter: ");
  Serial.println(Rcounter);
  if (Rcounter == Rows) {
    Serial.println("Reached Target Rows. Done!!");
    lcd.setCursor(0, 1);
    lcd.print(DispTxt[7][Lge]);  // reached the end
    PlayFinish();
  }
}

void PlayDn()  // Play tone sequence down
{
  tone(buzzer, 1000);
  delay(600);
  noTone(buzzer);
  delay(100);
  tone(buzzer, 800);
  delay(600);
  noTone(buzzer);
  delay(100);
}

void PlayUp() {
  tone(buzzer, 800);
  delay(600);
  noTone(buzzer);
  delay(100);
  tone(buzzer, 1000);
  delay(600);
  noTone(buzzer);
  delay(100);
}

void PlayFinish() {
  for (int thisNote = 0; thisNote < notes * 2; thisNote = thisNote + 2) {

    // calculates the duration of each note
    divider = melody[thisNote + 1];
    if (divider > 0) {
      // regular note, just proceed
      noteDuration = (wholenote) / divider;
    } else if (divider < 0) {
      // dotted notes are represented with negative durations!!
      noteDuration = (wholenote) / abs(divider);
      noteDuration *= 1.5;  // increases the duration in half for dotted notes
    }

    // we only play the note for 90% of the duration, leaving 10% as a pause
    tone(buzzer, melody[thisNote], noteDuration * 0.9);

    // Wait for the specified duration before playing the next note.
    delay(noteDuration);

    // stop the waveform generation before the next note.
    noTone(buzzer);
  }
}
